@@ -8,13 +8,13 @@ use Time::Local;
 use POSIX qw(strftime);
 
 # Prerequisite
-use Date::Calc 5.0 qw(Add_Delta_Days Easter_Sunday Day_of_Week This_Year);
+use Date::Calc qw(Add_Delta_Days Easter_Sunday Day_of_Week This_Year);
 
 require Exporter;
 
 our @ISA       = qw(Exporter);
 our @EXPORT_OK = qw(holidays);
-our $VERSION   = '1.3.1';
+our $VERSION   = '1.8.2';
 
 sub holidays{
 	my %parameters = (
@@ -30,11 +30,14 @@ sub holidays{
 
 	# Aliases for holidays
 	#
-	# neuj	= Neujahr
-	# hl3k	= Dreikoenigstag
+	# neuj  = Neujahr
+	# hl3k  = Dreikoenigstag
 	# romo  = Rosenmontag
 	# fadi  = Faschingsdienstag
+	# asmi  = Aschermittwoch
+	# grdo  = Gruendonnerstag
 	# karf  = Karfreitag
+	# kars  = Karsamstag
 	# osts  = Ostersonntag
 	# ostm  = Ostermontag
 	# pfis  = Pfingstsonntag
@@ -43,19 +46,25 @@ sub holidays{
 	# fron  = Fronleichnam
 	# 1mai  = Maifeiertag
 	# 17ju  = Tag der deutschen Einheit (>= 1954, <= 1990)
-	# frie	= Augsburger Friedensfest
-	# mari	= Mariae Himmelfahrt
+	# frie  = Augsburger Friedensfest
+	# mari  = Mariae Himmelfahrt
 	# 3okt  = Tag der deutschen Einheit (>= 1990)
 	# refo  = Reformationstag
 	# alhe  = Allerheiligen
 	# buss  = Buss- und Bettag
+	# votr  = Volkstrauertag
+	# toso  = Totensonntag
+	# adv1  = 1. Advent
+	# adv2  = 2. Advent
+	# adv3  = 3. Advent
+	# adv4  = 4. Advent
 	# heil  = Heiligabend
 	# wei1  = 1. Weihnachtstag
 	# wei2  = 2. Weihnachtstag
 	# silv  = Silvester
 
 	# Aliases for German federal states
-	# See http://www.bundestag.de/info/wahlen/154/1545.html
+	# See http://www.bmi.bund.de/cae/servlet/contentblob/150560/publicationFile/13610/feiertage_de.pdf
 	#
 	# bw = Baden-Wuerttemberg
 	# by = Freistaat Bayern
@@ -80,8 +89,15 @@ sub holidays{
 	my %holidays;
 	# Common holidays througout Germany
 	# 17ju/3okt are added to the list later, during date calculation
-	@{$holidays{'common'}} = qw(neuj karf osts ostm 1mai 
-		pfis pfim himm wei1 wei2);
+	@{$holidays{'common'}} = qw(neuj karf ostm 1mai
+		pfim himm wei1 wei2);
+
+	# 500th reformation day in 2017 will be a common federal holiday
+	# Deep link to Wikipedia as of today, 2016-01-06:
+	# https://de.wikipedia.org/w/index.php?title=Reformationstag&oldid=148889526#Bundesweiter_Feiertag_2017_.28einmalig.29
+	if (2017 == $year){
+		push @{$holidays{'common'}}, 'refo';
+	}
 
 	# Now the extra holidays for the federal states.
 	# As if things weren't bad enough, some holidays are only valid
@@ -95,11 +111,11 @@ sub holidays{
 	@{$holidays{'by'}} = qw(hl3k fron alhe);
 
 	# Extras for Berlin
-	@{$holidays{'bl'}} = qw();
-	
+	@{$holidays{'be'}} = qw();
+
 	# Extras for Brandenburg
-	@{$holidays{'bb'}} = qw(refo);
-	
+	@{$holidays{'bb'}} = qw(osts pfis refo);
+
 	# Extras for Bremen
 	@{$holidays{'hb'}} = qw();
 
@@ -132,7 +148,7 @@ sub holidays{
 
 	# Extras for Schleswig-Holstein
 	@{$holidays{'sh'}} = qw();
-	
+
 	# Extras for Thueringen
 	@{$holidays{'th'}} = qw(refo);
 
@@ -182,21 +198,41 @@ sub holidays{
 
 	# Holidays relative to Easter
 	#
+	# Fat Thursday = Easter Sunday minus 52 days
+	my ($j_weib, $m_weib, $t_weib) =
+		Date::Calc::Add_Delta_Days($year, $month, $day, -52);
+	$holiday{'weib'} = _date2timestamp($j_weib, $m_weib, $t_weib);
+
 	# Carnival Monday = Easter Sunday minus 48 days
 	my ($j_romo, $m_romo, $t_romo) =
 		Date::Calc::Add_Delta_Days($year, $month, $day, -48);
 	$holiday{'romo'} = _date2timestamp($j_romo, $m_romo, $t_romo);
-	
+
 	# Shrove Tuesday = Easter Sunday minus 47 days
 	my ($j_fadi, $m_fadi, $t_fadi) =
 		Date::Calc::Add_Delta_Days($year, $month, $day, -47);
 	$holiday{'fadi'} = _date2timestamp($j_fadi, $m_fadi, $t_fadi);
 
+	#  Ash Wednesday = Easter Sunday minus 46 days
+	my ($j_asmi, $m_asmi, $t_asmi) =
+		Date::Calc::Add_Delta_Days($year, $month, $day, -46);
+	$holiday{'asmi'} = _date2timestamp($j_asmi, $m_asmi, $t_asmi);
+
+	# Maundy Thursday = Easter Sunday minus 3 days
+	my ($j_grdo, $m_grdo, $t_grdo) =
+		Date::Calc::Add_Delta_Days($year, $month, $day, -3);
+	$holiday{'grdo'} = _date2timestamp($j_grdo, $m_grdo, $t_grdo);
+
 	# Good Friday = Easter Sunday minus 2 days
 	my ($j_karf, $m_karf, $t_karf) =
 		Date::Calc::Add_Delta_Days($year, $month, $day, -2);
 	$holiday{'karf'} = _date2timestamp($j_karf, $m_karf, $t_karf);
 
+	# Holy Saturday = Easter Sunday minus 1 day
+	my ($j_kars, $m_kars, $t_kars) =
+		Date::Calc::Add_Delta_Days($year, $month, $day, -1);
+	$holiday{'kars'} = _date2timestamp($j_kars, $m_kars, $t_kars);
+
 	# Easter Sunday is just that
 	$holiday{'osts'} = _date2timestamp($year, $month, $day);
 
@@ -241,6 +277,14 @@ sub holidays{
 		Date::Calc::Add_Delta_Days($year, 12, $tempdate, -32);
 	$holiday{'buss'} = _date2timestamp($j_buss, $m_buss, $t_buss);
 
+	# store the sundays in advent and the two sundays before
+	my $sc=0;
+	foreach my $name ("adv4", "adv3", "adv2", "adv1", "toso", "votr") {
+		$holiday{$name} = _date2timestamp(
+			Date::Calc::Add_Delta_Days($year, 12, $tempdate, 7*$sc--)
+		);
+	}
+
 	# Build list for returning
 	#
 	my %holidaylist;
@@ -334,9 +378,13 @@ The module knows about the following holidays:
 
   neuj  Neujahr                     New Year's day
   hl3k  Hl. 3 Koenige               Epiphany
+  weib  Weiberfastnacht             Fat Thursday
   romo  Rosenmontag                 Carnival monday
   fadi  Faschingsdienstag           Shrove tuesday
+  asmi  Aschermittwoch              Ash wednesday
+  grdo  Gruendonnerstag             Maundy Thursday
   karf  Karfreitag                  Good friday
+  kars  Karsamstag                  Holy Saturday
   osts  Ostersonntag                Easter sunday
   ostm  Ostermontag                 Easter monday
   pfis  Pfingstsonntag              Whit sunday
@@ -351,6 +399,12 @@ The module knows about the following holidays:
   refo  Reformationstag             Reformation day
   alhe  Allerheiligen               All hallows day
   buss  Buss- und Bettag            Penance day
+  votr  Volkstrauertag              Remembrance Day, German Style
+  toso  Totensonntag                Sunday in commemoration of the dead
+  adv1  1. Advent                   1st sunday in advent
+  adv2  2. Advent                   2nd sunday in advent
+  adv3  3. Advent                   3rd sunday in advent
+  adv4  4. Advent                   4th sunday in advent
   heil  Heiligabend                 Christmas eve
   wei1  1. Weihnachtstag            Christmas
   wei2  2. Weihnachtstag            Christmas
@@ -479,10 +533,10 @@ day is a holiday, too. Exclude weekends and return the date list in human
 readable format:
 
   my $feiertage_ref = holidays( WHERE    => ['common', 'he'],
-				FORMAT   => "%a, %d.%m.%Y"
-				WEEKENDS => 0,
+                                FORMAT   => "%a, %d.%m.%Y",
+                                WEEKENDS => 0,
                                 YEAR     => 2004,
-				ADD      => ['heil', 'silv', 'mari']);
+                                ADD      => ['heil', 'silv', 'mari']);
 
 =head1 PREREQUISITES
 
@@ -495,6 +549,11 @@ If you run into a miscalculation, need some sort of feature or an additional
 holiday, or if you know of any new changes to our funky holiday situation, 
 please drop the author a note.
 
+Patches are welcome. If you can, please fork the project on I<github> to
+submit your change:
+
+  http://github.com/mschmitt/Date-Holidays-DE
+
 I<Tag der Deutschen Einheit> was moved from June 17th to October 3rd in 1990
 and is therefore listed twice when calculating holidays for 1990. This is 
 not a bug. Awareness for this was introduced in Version 1.1.
